@@ -40,8 +40,7 @@ import ipywidgets as widgets
 import typing_extensions
 from typing_extensions import Literal, Protocol
 
-from . import patch  # noqa: F401
-from . import _version
+from . import _version, patch  # noqa: F401
 
 __version__ = _version.__version__
 
@@ -111,6 +110,28 @@ def suppress_events():
 widgets.Widget.element = classmethod(element)
 
 
+def _event_handler_exception_wrapper(f):
+    """Wrap an event handler to catch exceptions and put them in a reacton context.
+
+    This allows a component to catch the exception of a direct child"""
+    rc = get_render_context()
+    context = rc.context
+    assert context is not None
+
+    def wrapper(*args, **kwargs):
+        try:
+            f(*args, **kwargs)
+        except Exception as e:
+            assert context is not None
+            # because widgets don't have a context, but are a child of a component
+            # we add it to exceptions_children, not exception_self
+            # this allows a component to catch the exception of a direct child
+            context.exceptions_children.append(e)
+            rc.force_update()
+
+    return wrapper
+
+
 def join_key(parent_key, key):
     return f"{parent_key}{key}"
 
@@ -347,11 +368,12 @@ class Element(Generic[W]):
 
     def _add_widget_event_listener(self, widget: widgets.Widget, name: str, callback: Callable):
         target_name = name[3:]
+        callback_exception_safe = _event_handler_exception_wrapper(callback)
 
         def on_change(change):
             if are_events_supressed():
                 return
-            callback(change["new"])
+            callback_exception_safe(change["new"])
 
         self._callback_wrappers[callback] = on_change
         widget.observe(on_change, target_name)
@@ -1257,13 +1279,14 @@ class _RenderContext:
             # call the function, and recurse into, until we hit leafs
             # find a context from previous reconsolidation phase, or otherwise the previous render run
             context_previous = context.children_next.get(key)
+
             if context_previous is None:
                 context_previous = context.children.get(key)
             parent_context = context
             del context
             if context_previous is not None:
                 # We could reuse the same context
-                if context_previous.root_element is None:
+                if context_previous.root_element is None and context_previous.root_element_next is None:
                     # this happens when we already created a context (with state) using state_set()
                     context = context_previous
                     logger.debug("Render: Previous element was None, so we reuse the ComponentContext")
@@ -1318,7 +1341,7 @@ class _RenderContext:
                             if traceback.tb_next:  # is there an error inside the call
                                 self.tracebacks.append(traceback.tb_next)
                         self.tracebacks.append(el.traceback)
-                    logger.error("Component %r raised exception %r", el.component, e)
+                    logger.exception("Component %r raised exception %r", el.component, e)
                     context.exceptions_self.append(e)
                     self._rerender_needed_reason = "Exception ocurred during render"
                     self._rerender_needed = True
@@ -1337,19 +1360,25 @@ class _RenderContext:
                     if el.is_shared:
                         # TODO: why do the tests pass if we comment the next line out
                         self._shared_elements_next.remove(el)
-                # if we had an exception, we allow for LESS hooks calls, since the render body might not be executed completely
-                if not ((context.effect_index == len(context.effects)) or (context.exceptions_self and context.effect_index <= len(context.effects))):
-                    raise RuntimeError(
-                        f"Previously render had {len(context.effects)} effects, this run {context.effect_index} "
-                        f"(in element/component: {el}/{el.component}). "
-                        "Are you using conditional hooks?"
-                    )
-                if not ((context.memo_index == len(context.memo)) or (context.exceptions_self and context.memo_index <= len(context.memo))):
-                    raise RuntimeError(
-                        f"Previously render had {len(context.memo)} calls to use_memo, this run {context.memo_index} "
-                        f"(in element/component: {el}/{el.component}). "
-                        "Are you using conditional hooks?"
-                    )
+                try:
+                    # if we had an exception, we allow for LESS hooks calls, since the render body might not be executed completely
+                    if not ((context.effect_index == len(context.effects)) or (context.exceptions_self and context.effect_index <= len(context.effects))):
+                        raise RuntimeError(
+                            f"Previously render had {len(context.effects)} effects, this run {context.effect_index} "
+                            f"(in element/component: {el}/{el.component}). "
+                            "Are you using conditional hooks?"
+                        )
+                    if not ((context.memo_index == len(context.memo)) or (context.exceptions_self and context.memo_index <= len(context.memo))):
+                        raise RuntimeError(
+                            f"Previously render had {len(context.memo)} calls to use_memo, this run {context.memo_index} "
+                            f"(in element/component: {el}/{el.component}). "
+                            "Are you using conditional hooks?"
+                        )
+                except RuntimeError as e:
+                    logger.exception("Exception in hook count check")
+                    context.exceptions_self.append(e)
+                    self._rerender_needed_reason = "Exception ocurred during render (hook count check)"
+                    self._rerender_needed = True
                 # only expose to parent when no error occurs
                 context.parent.children_next[key] = context
                 # drop all children from the previous render run (this render phase)
