@@ -4,6 +4,7 @@ import unittest.mock
 from concurrent.futures import ThreadPoolExecutor
 from typing import Callable, Generic, List, Optional, Tuple, TypeVar, cast
 
+import ipyvuetify
 import ipywidgets
 import ipywidgets as widgets
 import numpy as np
@@ -13,8 +14,8 @@ import traitlets
 import reacton as react
 from reacton.core import component, use_effect
 
+from . import bqplot  # noqa: F401
 from . import logging  # noqa: F401
-from . import bqplot
 from . import ipyvuetify as v
 from . import ipywidgets as w
 from .core import ipywidget_version_major
@@ -1675,6 +1676,95 @@ def test_exception_handler_exception():
     rc.close()
 
 
+def test_exception_handler_in_event_widget():
+    # when the exception handling raises an exception, rerender again and bubble up
+    exception = None
+    clear = None
+
+    @react.component
+    def Container():
+        nonlocal clear, exception
+        exception, clear = react.use_exception()
+
+        def on_click():
+            raise Exception("in event handler: on_click")
+
+        def on_description(change):
+            raise Exception("in event handler: on_description")
+
+        return w.Button(on_click=on_click, on_description=on_description)
+
+    box, rc = react.render(Container(), handle_error=False)
+    assert clear is not None
+    rc.find(ipywidgets.Button).widget.click()
+    assert exception is not None
+    assert str(exception) == "in event handler: on_click"
+    rc.find(ipywidgets.Button).widget.description = "new description"
+    assert str(exception) == "in event handler: on_description"
+    # because the Handler failed to catch the error, the core library does it
+    assert not rc._is_rendering
+
+    rc.close()
+
+
+def test_exception_handler_in_event_widget_vue():
+    # when the exception handling raises an exception, rerender again and bubble up
+    exception = None
+    clear = None
+
+    @react.component
+    def Container():
+        nonlocal clear, exception
+        exception, clear = react.use_exception()
+
+        def on_click():
+            raise Exception("in event handler: on_click")
+
+        btn = v.Btn(children=[str(exception)])
+        v.use_event(btn, "click", lambda *_ignore: on_click())
+        return btn
+
+    box, rc = react.render(Container(), handle_error=False)
+    assert clear is not None
+    rc.find(ipyvuetify.Btn).widget.fire_event("click", {})
+    assert exception is not None
+    assert str(exception) == "in event handler: on_click"
+    # because the Handler failed to catch the error, the core library does it
+    assert not rc._is_rendering
+
+    rc.close()
+
+
+def test_exception_handler_due_to_use_memo():
+    @react.component
+    def Child():
+        value, set_value = react.use_state(0)
+        for i in range(value):
+            react.use_memo(lambda: 1)  # this will raise an exception if value changes
+        return w.IntSlider(value=value, on_value=set_value)
+
+    @react.component
+    def Container():
+        exception, clear = react.use_exception()
+        if not exception:
+            return Child()
+        else:
+            return w.Button(on_click=clear, description=str(exception))
+
+    box, rc = react.render(Container(), handle_error=False)
+    assert clear is not None
+    assert rc.find(ipywidgets.IntSlider).widget.value == 0
+    rc.find(ipywidgets.IntSlider).widget.value = 1
+    rc.find(ipywidgets.Button).assert_empty()
+    rc.find(ipywidgets.IntSlider).widget.value = 0
+    rc.find(ipywidgets.Button).widget.click()
+    rc.find(ipywidgets.Button).assert_empty()
+    assert rc.find(ipywidgets.IntSlider).widget.value == 0
+    assert not rc._is_rendering
+
+    rc.close()
+
+
 def test_recover_exception_in_reconcilliate():
     set_fail = None
 
@@ -2319,6 +2409,43 @@ def test_rerender_component_switch(Container):
     rc.close()
 
 
+def test_rerender_component_switch_memo(Container):
+    # this tests when during the initial render phase, we have
+    # two render loops, where we switch a component, that
+    # should not re-used the context of the previous render loop
+    # this is something we check by having less use_memo's the
+    # second time
+    @react.component
+    def Component1():
+        react.use_memo(lambda: 1)
+        react.use_memo(lambda: 1)
+        return w.Button(description="1")
+
+    @react.component
+    def Component2():
+        react.use_memo(lambda: 1)
+        return w.Button(description="2")
+
+    set_value = None
+
+    @react.component
+    def Test():
+        nonlocal set_value
+        value, set_value = react.use_state(1)
+        set_value(2)
+        with Container() as main:
+            if value == 1:
+                Component1()
+            else:
+                Component2()
+        return main
+
+    box, rc = react.render(Test(), handle_error=False)
+    assert set_value is not None
+    rc.find(widgets.Button).assert_matches(description="2")
+    rc.close()
+
+
 def test_set_state_during_close():
     @react.component
     def Test():
